package com.yizhi.lecturer.application.feign;


import com.baomidou.mybatisplus.plugins.Page;

import com.yizhi.lecturer.application.vo.*;
import com.yizhi.lecturer.application.vo.domain.LecturerVo;
import com.yizhi.lecturer.application.vo.domain.LecturerDicVo;
import com.yizhi.lecturer.application.vo.domain.LecturerResponseVo;
import org.springframework.cloud.openfeign.FeignClient;
import org.springframework.web.bind.annotation.*;

import java.util.List;

/**
 * @author lingye
 * @date 2019-4-17
 */
@FeignClient(name = "lecturer", contextId = "LecturerClient")
public interface LecturerClient {
    /**
     * 获取用户
     *
     * @param code
     * @return
     */
    @GetMapping("/lecturerdic/list/code")
    List<LecturerDicVo> getLecturerDicList(@RequestParam(name = "code") String code);

    /**
     * 添加讲师
     *
     * @param lecturerVO
     * @return
     * @date 2019-4-24
     */
    @PostMapping("/lecturer/add")
    LecturerVO saveLecturer(@RequestBody LecturerVO lecturerVO);

    /**
     * 修改讲师
     *
     * @param lecturerVO
     * @return
     */
    @PostMapping("/lecturer/update")
    LecturerVO updateLecturer(@RequestBody LecturerVO lecturerVO);

    @PostMapping("/lecturer/upOrDown")
    LecturerResponseVo upOrDown(@RequestBody LecturerVo lecturerVo);

    /**
     * 查询讲师的详情
     *
     * @param id
     * @return
     */
    @GetMapping("/lecturer/get")
    LecturerVO queryLecturer(@RequestParam("id") Long id);

    /**
     * 交银康联接口
     * @param lecturerApiVoParam
     * @return
     */
    @PostMapping("/lecturer/find")
    Page<LecturerApiVo> findLecturer(@RequestBody LecturerApiVoParam lecturerApiVoParam);


    @GetMapping("/lecturer/detail")
    public LecturerVO queryLecturerByLecturerId(@RequestParam("id") Long id);

    /**
     * 查询讲师列表
     *
     * @param name
     * @param mobile
     * @param sourceCode 来源code
     * @param pageNo
     * @param pageSize
     * @return
     */
    @GetMapping("/lecturer/list")
    Page<LecturerListVO> queryLecturer(@RequestParam(name = "name", required = false) String name,
                                       @RequestParam(name = "mobile", required = false) String mobile,
                                       @RequestParam(name = "sourceCode", required = false) String sourceCode,
                                       @RequestParam(name = "pageNo", defaultValue = "1") Integer pageNo,
                                       @RequestParam(name = "pageSize", defaultValue = "20") Integer pageSize);

    /**
     * 导入讲师excel
     *
     * @return
     */
    @GetMapping("/lecturer/export")
    List<LecturerListVO> exportLecturer(@RequestParam(name = "keywords", required = false) String keywords,
                                        @RequestParam(name = "sourceCode", required = false) String sourceCode,
                                        @RequestParam(name = "mobile", required = false) String mobile);

    /**
     * 排序
     *
     * @param lecturerSortVO
     * @return
     */
    @PostMapping("/lecturer/sortLecturer")
    boolean sortLecturer(@RequestBody LecturerSortVO lecturerSortVO);

    /**
     * 删除讲师
     *
     * @param lecturerSetVO
     * @return
     */
    @PostMapping("/lecturer/del")
    LecturerResponseVo delLecturer(@RequestBody LecturerSetVO lecturerSetVO);

    /**
     * 通过id获取讲师
     *
     * @param id
     * @return
     */
    @GetMapping("/lecturer/view")
    LecturerVo getLecturerById(@RequestParam(name = "id") Long id);

    /**
     * 获取模块的列表
     *
     * @param lecturerTemplateVO
     * @return
     */
    @PostMapping("/lecturer/query")
    Page<TemplateLecturerVO> queryLecturerByTemplateId(@RequestBody(required = false) LecturerTemplateVO lecturerTemplateVO);

    /**
     * 获取关联列表
     *
     * @param id
     * @return
     */
    @GetMapping("/lecturer/relation/get")
    List<RelationVO> getRelationsById(@RequestParam(name = "id", required = true) Long id);

    /**
     * 获取讲师关联的课程/培训
     *
     * @param lecturerVO
     * @return
     */
    @PostMapping("/lecturer/relation/update")
    boolean updateRelation(@RequestBody LecturerVO lecturerVO);


    /**
     * 获取讲师课程和培训
     *
     * @param id
     * @param pageNo
     * @param pageSize
     * @return
     */
    @GetMapping("/lecturer/relation/query")
    Page<RelationVOList> getRelationListByLecturerId(@RequestParam(name = "id", required = false) Long id,
                                                     @RequestParam(name = "pageNo", defaultValue = "1") Integer pageNo,
                                                     @RequestParam(name = "pageSize", defaultValue = "20") Integer pageSize);


    /**
     * 新增讲师基本信息
     *
     * @param lecturerInfoVO
     * @return
     */
    @PostMapping(value = "/info/add")
    LecturerInfoVO add(@RequestBody LecturerInfoVO lecturerInfoVO);

    /**
     * 修改讲师基本信息
     *
     * @param lecturerInfoVO
     * @return
     */
    @PostMapping(value = "/info/update")
    LecturerInfoVO update(@RequestBody LecturerInfoVO lecturerInfoVO);

    /**
     * 获取基本信息
     *
     * @return
     */
    @GetMapping(value = "/info/get")
    LecturerInfoVO get();

    /**
     * 评分导入
     * @param model
     * @return
     */
    @PostMapping("/lecturer/import/score")
    String importScore(@RequestBody FileUploadModel model);

    /**
     * 评级导入
     * @param model
     * @return
     */
    @PostMapping("/lecturer/import/grade")
    String importGrade(@RequestBody FileUploadModel model);

    /**
     * 讲师库课程导入
     * @param model
     * @return
     */
    @PostMapping(value = "/lecturer/import/course")
    String importCourse(@RequestBody FileUploadModel model);

    /**
     * 修改评级和评分
     * @param lecturerVo
     * @return
     */
    @PostMapping(value = "/lecturer/grade")
    Boolean updateGrade(@RequestBody LecturerVo lecturerVo);
}
