package com.yizhi.application.Controller;


import com.baomidou.mybatisplus.plugins.Page;
import com.yizhi.application.domain.Lecturer;
import com.yizhi.application.domain.LecturerResponse;
import com.yizhi.lecturer.application.enums.DeleteFlag;
import com.yizhi.application.service.ILecturerService;
import com.yizhi.lecturer.application.vo.*;
import com.yizhi.lecturer.application.vo.domain.LecturerResponseVo;
import com.yizhi.lecturer.application.vo.domain.LecturerVo;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiParam;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.List;

/**
 * <p>
 * 讲师详情表 前端控制器
 * </p>
 *
 * @author lingye
 * @since 2019-04-17
 */
@Api(tags = "讲师库", value = "讲师库")
@RestController
@RequestMapping("/lecturer")
public class LecturerController {

    @Autowired
    private ILecturerService lecturerService;

    /**
     * 新增讲师
     *
     * @param lecturerVO
     * @return
     */
    @PostMapping("/add")
    public LecturerVO saveLecturer(@RequestBody LecturerVO lecturerVO) {
        return lecturerService.addLecturer(lecturerVO);
    }

    /**
     * 修改讲师
     *
     * @param lecturerVO
     * @return
     */
    @PostMapping("/update")
    public LecturerVO updateLecturer(@RequestBody LecturerVO lecturerVO) {
        boolean updateLecturer = lecturerService.updateLecturer(lecturerVO);
        if (updateLecturer) {
            return queryLecturer(lecturerVO.getId());
        } else {
            return null;
        }
    }

    /**
     * 讲师库的停用/
     * @param lecturer
     * @return
     */
    @PostMapping("/upOrDown")
    public LecturerResponseVo upOrDown(@RequestBody Lecturer lecturer) {
        LecturerResponse list = lecturerService.upOrDownLecturer(lecturer);
        LecturerResponseVo list2 = new LecturerResponseVo();
        BeanUtils.copyProperties(list, list2);
        return list2;
    }

    /**
     * 查询讲师列表
     *
     * @param id
     * @return
     */
    @GetMapping("/get")
    public LecturerVO queryLecturer(@RequestParam("id") Long id) {
        return lecturerService.queryLecturerById(id);
    }

    /**
     * 交银康联接口
     * @param lecturerApiVoParam
     * @return
     */
    @PostMapping("/find")
    Page<LecturerApiVo> findLecturer(@RequestBody LecturerApiVoParam lecturerApiVoParam){
        return lecturerService.findLecturer(lecturerApiVoParam);
    }

    /**
     * 学员端-讲师详情
     *
     * @param id
     * @return
     */
    @GetMapping("/detail")
    public LecturerDetailVO queryLecturerByLecturerId(@RequestParam("id") Long id) {
        return lecturerService.queryLecturerByLecturerId(id);
    }

    @GetMapping("/list")
    public Page<LecturerListVO> queryLecturer(@RequestParam(name = "name", required = false) String name,
                                              @RequestParam(name = "mobile", required = false) String mobile,
                                              @RequestParam(name = "sourceCode", required = false) String sourceCode,
                                              @RequestParam(name = "pageNo", defaultValue = "1") Integer pageNo,
                                              @RequestParam(name = "pageSize", defaultValue = "20") Integer pageSize
    ) {
        return lecturerService.queryLecturer(name, mobile, sourceCode, pageNo, pageSize);
    }

    /**
     * 导出讲师列表
     *
     * @return
     */
    @GetMapping("/export")
    public List<LecturerListVO> exportLecturer(@ApiParam(name = "keywords", value = "讲师名称/课程名称") @RequestParam(name = "keywords", required = false) String keywords,
                                               @ApiParam(name = "sourceCode", value = "讲师来源inner 内部/outer 外部") @RequestParam(name = "sourceCode", required = false) String sourceCode,
                                               @ApiParam(name = "mobile", value = "手机号码") @RequestParam(name = "mobile", required = false) String mobile
    ) {
        return lecturerService.exportLecturer(keywords, sourceCode, mobile);
    }

    /**
     * 排序
     *
     * @param lecturerSortVO
     * @return
     */
    @PostMapping("/sortLecturer")
    public boolean sortLecturer(@RequestBody LecturerSortVO lecturerSortVO) {
        return lecturerService.sortLecturer(lecturerSortVO);
    }

    /**
     * 删除讲师库
     *
     * @param lecturerSetVO
     * @return
     */
    @PostMapping("/del")
    public LecturerResponseVo delLecturer(@RequestBody LecturerSetVO lecturerSetVO) {
        LecturerResponse lecturerResponse = lecturerService.delLecturer(lecturerSetVO.getIds());
        LecturerResponseVo lecturerResponseVo1 = new LecturerResponseVo();
        BeanUtils.copyProperties(lecturerResponse, lecturerResponseVo1);
        return lecturerResponseVo1;
    }

    /**
     * 通过id获取讲师
     *
     * @param id
     * @return
     */
    @GetMapping("/view")
    public LecturerVo getLecturerById(@RequestParam(name = "id") Long id) {
        Lecturer lecturer = lecturerService.getLecturerById(id);
        LecturerVo lecturerVo1 = new LecturerVo();
        BeanUtils.copyProperties(lecturer, lecturerVo1);
        return lecturerVo1;
    }


    /**
     * 移动模块的添加
     *
     * @param lecturerTemplateVO
     * @return
     */
    @PostMapping("/query")
    public Page<TemplateLecturerVO> queryLecturerByTemplateId(@RequestBody(required = false) LecturerTemplateVO lecturerTemplateVO) {

        Integer pageNo = lecturerTemplateVO.getPageNo() != null ? lecturerTemplateVO.getPageNo() : DeleteFlag.NO.ordinal();
        Integer pageSize = lecturerTemplateVO.getPageSize() != null ? lecturerTemplateVO.getPageSize() : 20;
        return lecturerService.queryLecturerByTemplateId(lecturerTemplateVO.getName(),
                pageNo, pageSize
                , lecturerTemplateVO.getIds());
    }

    /**
     * 修改评级和积分
     * @param lecturer
     * @return
     */
    @PostMapping("/grade")
    public Boolean updateGrade(@RequestBody Lecturer lecturer) {
        return lecturerService.updateById(lecturer);
    }
}

