package com.yizhi.application.Controller;


import com.alibaba.fastjson.JSON;
import com.yizhi.application.orm.id.IdGenerator;
import com.yizhi.application.orm.util.DomainConverter;
import com.yizhi.core.application.context.ContextHolder;
import com.yizhi.core.application.context.RequestContext;
import com.yizhi.application.service.ILecturerImportService;
import com.yizhi.application.service.impl.LecturerClassificationServiceImpl;
import com.yizhi.core.application.context.TaskContext;
import com.yizhi.core.application.file.constant.FileConstant;
import com.yizhi.core.application.file.domain.FileInfo;
import com.yizhi.core.application.file.domain.Template;
import com.yizhi.core.application.file.domain.TemplateColumn;
import com.yizhi.core.application.file.exp.ExcelExportor;
import com.yizhi.core.application.file.imp.AbstractImporter;
import com.yizhi.core.application.file.imp.ImportCallBack;
import com.yizhi.core.application.file.util.OssUpload;
import com.yizhi.core.application.log.LogEventPublisher;
import com.yizhi.core.application.log.TaskLogEvent;
import com.yizhi.core.application.task.TaskExecutor;
import com.yizhi.core.application.task.TaskHandler;
import com.yizhi.lecturer.application.vo.FileUploadModel;
import com.yizhi.lecturer.application.vo.LecturerCourseImport;
import com.yizhi.lecturer.application.vo.LecturerGradeImport;
import com.yizhi.lecturer.application.vo.LecturerScoreImport;
import com.yizhi.system.application.model.ImportModel;
import com.yizhi.util.application.clazz.ClassUtil;
import com.yizhi.util.application.date.DateUtil;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang.StringUtils;
import org.apache.poi.hssf.usermodel.HSSFRow;
import org.apache.poi.hssf.usermodel.HSSFSheet;
import org.apache.poi.hssf.usermodel.HSSFWorkbook;
import org.apache.poi.ss.usermodel.Cell;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

/**
 * <p>
 * 讲师详情表 前端控制器
 * </p>
 *
 * @author lingye
 * @since 2019-04-17
 */
@Api(tags = "讲师库导入", value = "讲师库导入")
@RestController
@RequestMapping("/lecturer/import")
public class LecturerImportController {

    private static final Logger logger = LoggerFactory.getLogger(LecturerClassificationServiceImpl.class);

    @Autowired
    IdGenerator idGenerator;

    @Autowired
    DomainConverter converter;

    @Autowired
    private ILecturerImportService lecturerImportService;

    @Autowired
    LogEventPublisher publisher;

    @Autowired
    TaskExecutor taskExecutor;


    @ApiOperation(value = "评分导入", notes = "评分导入")
    @PostMapping(value = "/score")
    public String importScore(@RequestBody FileUploadModel model) {

        logger.info("导入文件信息=====" + JSON.toJSONString(model));
        String taskLogSerialNo = "LECTURER_SCORE-" + DateUtil.getYMDHMSS();
        try {
            Long taskId = idGenerator.generate();
            RequestContext context = ContextHolder.get();
            Long companyId = context.getCompanyId();
//            Long companyId = 1314L;
            Long siteId = context.getSiteId();
//            Long siteId = 1314L;
            Long accountId = context.getAccountId();
//            Long accountId = 1314L;

            TaskContext taskContext = new TaskContext(taskId, taskLogSerialNo, "讲师库批量导入评分", accountId, new Date(), siteId,
                    companyId);
            ImportModel importModel = (ImportModel) converter.toDO(context, ImportModel.class);
            importModel.setTaskId(taskId);
            ImportCallBack callBack = new ImportCallBack() {
                @Override
                public Boolean exec(TaskHandler taskHandler, List dataObjs) {
                    importModel.setData(dataObjs);
                    List<LecturerScoreImport> models = lecturerImportService.importScore(importModel, 1,context);
                    // 所有的信息要上传到oss服务器
                    ExcelForAccountImportExporter<LecturerScoreImport> exporter = new ExcelForAccountImportExporter<>(
                            createExpTemplate(1), models);
                    exporter.bindContext(taskContext);
                    exporter.bindPublisher(publisher);
                    exporter.handle();
                    return true;
                }
            };
            AbstractImporter importer = AbstractImporter.build(model.getFileUrl(), null, model.getFileName(), callBack,
                    createScoreImpTemplate(), LecturerScoreImport.class);
            importer.bindContext(taskContext);
            importer.bindPublisher(publisher);
            taskExecutor.asynExecute(importer);
        } catch (Exception e) {
            logger.error("", e);
//            return Response.fail(ReturnCode.SERVICE_UNAVAILABLE);
        }
        return taskLogSerialNo;
    }

    @ApiOperation(value = "评级导入", notes = "评级导入")
    @PostMapping(value = "/grade")
    public String importGrade(@RequestBody FileUploadModel model) {

        logger.info("导入文件信息=====" + JSON.toJSONString(model));
        String taskLogSerialNo = "LECTURER_GRADE-" + DateUtil.getYMDHMSS();
        try {
            Long taskId = idGenerator.generate();
            RequestContext context = ContextHolder.get();
            Long companyId = context.getCompanyId();
//            Long companyId = 1314L;
            Long siteId = context.getSiteId();
//            Long siteId = 1314L;
            Long accountId = context.getAccountId();
//            Long accountId = 1314L;

            TaskContext taskContext = new TaskContext(taskId, taskLogSerialNo, "讲师库批量导入评级", accountId, new Date(), siteId,
                    companyId);
            ImportModel importModel = (ImportModel) converter.toDO(context, ImportModel.class);
            importModel.setTaskId(taskId);
            ImportCallBack callBack = new ImportCallBack() {
                @Override
                public Boolean exec(TaskHandler taskHandler, List dataObjs) {
                    importModel.setData(dataObjs);
                    List<LecturerScoreImport> models = lecturerImportService.importScore(importModel, 2,context);
                    ExcelForAccountImportExporter<LecturerGradeImport> exporter = new ExcelForAccountImportExporter<>(
                            createExpTemplate(2), models);
                    exporter.bindContext(taskContext);
                    exporter.bindPublisher(publisher);
                    exporter.handle();
                    return true;
                }
            };
            AbstractImporter importer = AbstractImporter.build(model.getFileUrl(), null, model.getFileName(), callBack,
                    createScoreImpTemplate(), LecturerScoreImport.class);
            importer.bindContext(taskContext);
            importer.bindPublisher(publisher);
            taskExecutor.asynExecute(importer);
        } catch (Exception e) {
            logger.error("", e);
//            return Response.fail(ReturnCode.SERVICE_UNAVAILABLE);
        }
        return taskLogSerialNo;
    }

    @ApiOperation(value = "课程导入", notes = "课程导入")
    @PostMapping(value = "/course")
    public String importCourse(@RequestBody FileUploadModel model) {

        logger.info("导入文件信息=====" + JSON.toJSONString(model));
        String taskLogSerialNo = "LECTURER_COURSE-" + DateUtil.getYMDHMSS();
        try {
            Long taskId = idGenerator.generate();
            RequestContext context = ContextHolder.get();
            Long companyId = context.getCompanyId();
//            Long companyId = 1314L;
            Long siteId = context.getSiteId();
//            Long siteId =1314L;
            Long accountId = context.getAccountId();
//            Long accountId = 1314L;

            TaskContext taskContext = new TaskContext(taskId, taskLogSerialNo, "讲师库批量导入关联课程", accountId, new Date(), siteId,
                    companyId);
            ImportModel importModel = (ImportModel) converter.toDO(context, ImportModel.class);
            importModel.setTaskId(taskId);
            ImportCallBack callBack = new ImportCallBack() {
                @Override
                public Boolean exec(TaskHandler taskHandler, List dataObjs) {
                    importModel.setData(dataObjs);
                    List<LecturerCourseImport> models = lecturerImportService.importCourse(importModel,context);
                    ExcelForAccountImportExporter<LecturerCourseImport> exporter = new ExcelForAccountImportExporter<>(
                            createExpTemplate(3), models);
                    exporter.bindContext(taskContext);
                    exporter.bindPublisher(publisher);
                    exporter.handle();
                    return true;
                }
            };
            AbstractImporter importer = AbstractImporter.build(model.getFileUrl(), null, model.getFileName(), callBack,
                    createCourseImpTemplate(), LecturerCourseImport.class);
            importer.bindContext(taskContext);
            importer.bindPublisher(publisher);
            taskExecutor.asynExecute(importer);
        } catch (Exception e) {
            logger.error("", e);
//            return Response.fail(ReturnCode.SERVICE_UNAVAILABLE);
        }
        return taskLogSerialNo;
    }


    class ExcelForAccountImportExporter<T> extends ExcelExportor<T> {

        public ExcelForAccountImportExporter(Template template, List data) {
            super(template, data);
        }

        @Override
        public FileInfo render() {
            FileInfo fileInfo = new FileInfo();
            fileInfo.setFileType(template.getFileType());
            // 创建HSSFWorkbook对象(excel的文档对象)
            HSSFWorkbook wb = new HSSFWorkbook();
            // 建立新的sheet对象（excel的表单）
            HSSFSheet sheet = wb.createSheet();
            // 在sheet里创建第一行，标题行
            HSSFRow row1 = sheet.createRow((int) 0);
            // 标题目录
            List<TemplateColumn> columns = template.getColumns();
            for (int i = 0; i < columns.size(); i++) {
                Cell cell = row1.createCell(i);
                cell.setCellValue(columns.get(i).getColumnName().toString());
            }
            // 行数据
           HSSFRow row2 = null;
            int passCount = 0;
            int failCount = 0;
            if (CollectionUtils.isNotEmpty(data)) {
                for (int j = 0; j < data.size(); j++) {
                    T t = data.get(j);
                    row2 = sheet.createRow((int) j + 1);
                    for (int m = 0; m < columns.size(); m++) {
                        TemplateColumn templateColumn = columns.get(m);
                        String cellValue = ClassUtil.invokeMethod(t, templateColumn.getColumn()).toString();
                        row2.createCell(m).setCellValue(cellValue);
                    }
                    if (StringUtils.isNotEmpty(
                            ClassUtil.invokeMethod(t, columns.get(columns.size() - 1).getColumn()).toString())) {
                        failCount++;
                    } else {
                        passCount++;
                    }
                }
            }
            String fileName = template.getFileName();
            if (StringUtils.isEmpty(fileName)) {
                fileName = "导出文件".concat(String.valueOf(System.currentTimeMillis())).concat(".xls");
            }
            fileInfo.setFileName(fileName);
            if (template.getBrowserExport()) {
                response.setHeader("Content-Disposition", "attachment; filename=" + fileName);
                response.setContentType("application/vnd.ms-excel");
                OutputStream output = null;
                try {
                    output = response.getOutputStream();
                    wb.write(output);
                    output.flush();
                    publishTaskEvent(TaskLogEvent.success(taskContext, "导入成功", null));
                } catch (IOException e) {
                    publishTaskEvent(TaskLogEvent.fail(taskContext, "文件导入异常:" + e.getMessage()));
                    logger.error("文件导入异常", e);
                } finally {
                    try {
                        output.close();
                    } catch (IOException e) {
                        logger.error("文件导入关闭流异常", e);
                    }
                }
            } else {
                String path = new StringBuffer().append(FileConstant.SAVE_PATH).append(File.separator).append(fileName)
                        .toString();
//	    	//文件保存位置
                File saveDir = new File(FileConstant.SAVE_PATH);
                if (!saveDir.exists()) {
                    saveDir.mkdir();
                }
                File file = new File(saveDir + File.separator + fileName);
                try {
                    FileOutputStream os = new FileOutputStream(file);
                    wb.write(os);
                    os.close();
                    // 阿里云返回url
                    String upLoadUrl = OssUpload.upload(path, fileName);// path, fileName
                    publishTaskEvent(TaskLogEvent.success(taskContext, "成功导入" + passCount + "条，失败" + failCount + "条",
                            upLoadUrl));
                    file.delete();
                } catch (Exception e) {
                    publishTaskEvent(TaskLogEvent.fail(taskContext, "写入过程中发生错误:" + e.getMessage()));
                    logger.error("写入过程中发生错误", e);
                }
            }
            return fileInfo;
        }
    }

    /**
     * 讲师评级
     *
     * @return
     */
    private Template createExpTemplate(Integer type) {
        Template template = new Template();
        if (type == 2) {
            template = createGradeImpTemplate();
        } else if (type == 1) {
            template = createScoreImpTemplate();
        } else if (type == 3) {
            template = createCourseImpTemplate();
        }
        if ( type == 3) {
            logger.info("课程创建模板");
            List<TemplateColumn> columns = template.getColumns();
            columns.add(new TemplateColumn("result", "结果", 6));
            template.setColumns(columns);
            template.setBrowserExport(false);
        } else {
            List<TemplateColumn> columns = template.getColumns();
            columns.add(new TemplateColumn("result", "结果", 4));
            template.setColumns(columns);
            template.setBrowserExport(false);
        }
        return template;
    }


    /**
     * 讲师评级模板
     *
     * @return
     */
    private Template createGradeImpTemplate() {
        Template template = new Template();
        List<TemplateColumn> columns = new ArrayList<>();
        columns.add(new TemplateColumn("code", "讲师编码", 1));
        columns.add(new TemplateColumn("name", "讲师姓名", 2));
        columns.add(new TemplateColumn("score", "讲师评级", 3));
        template.setColumns(columns);
        return template;
    }

    /**
     * 讲师评分模板
     *
     * @return
     */
    private Template createScoreExpTemplate() {
        Template template = createScoreImpTemplate();
        List<TemplateColumn> columns = template.getColumns();
        columns.add(new TemplateColumn("result", "结果", 4));
        template.setColumns(columns);
        template.setBrowserExport(false);
        return template;
    }

    /**
     * 讲师评分模板
     *
     * @return
     */
    private Template createScoreImpTemplate() {
        Template template = new Template();
        List<TemplateColumn> columns = new ArrayList<>();
        columns.add(new TemplateColumn("code", "讲师编码", 1));
        columns.add(new TemplateColumn("name", "讲师姓名", 2));
        columns.add(new TemplateColumn("score", "讲师评分", 3));
        template.setColumns(columns);
        return template;
    }

    /**
     * 课程导入
     *
     * @return
     */
    private Template createCourseImpTemplate() {
        Template template = new Template();
        List<TemplateColumn> columns = new ArrayList<>();
        columns.add(new TemplateColumn("code", "讲师编码", 1));
        columns.add(new TemplateColumn("name", "讲师姓名", 2));
        columns.add(new TemplateColumn("type", "课程类型", 3));
        columns.add(new TemplateColumn("courseCode", "课程编码", 4));
        columns.add(new TemplateColumn("courseName", "课程名称", 5));
        template.setColumns(columns);
        return template;
    }
}

