package com.yizhi.application.mapper;

import com.baomidou.mybatisplus.mapper.BaseMapper;
import com.baomidou.mybatisplus.plugins.Page;
import com.yizhi.application.domain.Lecturer;
import com.yizhi.lecturer.application.vo.*;
import org.apache.ibatis.annotations.Param;
import org.apache.ibatis.annotations.Select;
import org.springframework.stereotype.Repository;

import java.util.List;

/**
 * <p>
 * 讲师详情表 Mapper 接口
 * </p>
 *
 * @author lingye
 * @since 2019-04-17
 */
@Repository
public interface LecturerMapper extends BaseMapper<Lecturer> {
    /**
     * 查询讲师最后添加的sort
     *
     * @param siteId 站点id
     * @return
     */
    @Select("select IFNULL(max(sort),0) from lecturer where site_id =#{siteId}")
    int queryLecturerMaxSort(@Param("siteId") Long siteId);

    /**
     * 查询前一个sort
     *
     * @param siteId
     * @param sort
     * @return
     */

    Lecturer getLecturerBeforeSort(@Param("siteId") Long siteId, @Param("sort") int sort);

    /**
     * 查询后一个讲师的sort
     *
     * @param siteId
     * @param sort
     * @return
     */
    Lecturer getLecturerAfterSort(@Param("siteId") Long siteId, @Param("sort") int sort);


    /**
     * 查询列表
     *
     * @param siteId
     * @param sort
     * @return
     */
    List<Lecturer> getLecturerBeforeSortList(@Param("siteId") Long siteId, @Param("sort") int sort);

    /**
     * 讲师列表
     *
     * @param page
     * @param name
     * @param mobile
     * @param sourceCode 来源code
     * @param siteId
     * @return
     */
    List<LecturerListVO> getLecturerList(
            Page<LecturerListVO> page, @Param("name") String name,
            @Param("mobile") String mobile, @Param("sourceCode") String sourceCode, @Param("siteId") Long siteId);

    /**
     * 导出excel
     *
     * @param siteId
     * @return
     */
    List<LecturerListVO> exportLecturer(
            @Param("siteId") Long siteId,
            @Param("keywords") String keywords,
            @Param("sourceCode") String sourceCode,
            @Param("mobile") String mobile);

    /**
     * 根据id获取讲师详情
     *
     * @param id
     * @return
     */
    LecturerVO getLecturerVOById(@Param("id") Long id);

    /**
     * 查询模块列表
     *
     * @param page
     * @param name
     * @param siteId
     * @param ids
     * @return
     */
    List<TemplateLecturerVO> queryLecturerList(
            Page<TemplateLecturerVO> page, @Param("name") String name,
            @Param("siteId") Long siteId,
            @Param("ids") List<Long> ids);


    @Select("select IFNULL(max(substring(`code`,2)), '0') from lecturer where site_id =#{siteId}")
    String queryLecturerCodexSort(@Param("siteId") Long siteId);

    /**
     * 学员端- 讲师详情
     *
     * @param id
     * @return
     */
    LecturerDetailVO getStuLecturerDetail(@Param("id") Long id);

    /**
     * 查询讲师库
     * @param page
     * @param companyId
     * @param siteId
     * @param ids
     * @return
     */
    List<LecturerApiVo> selectLecturerList(@Param("page") Page<LecturerApiVo> page,
                                           @Param("companyId") Long companyId, @Param("siteId") Long siteId, @Param("ids") List<String> ids);
}