package com.yizhi.application.service;

import com.baomidou.mybatisplus.plugins.Page;
import com.yizhi.application.domain.Lecturer;
import com.baomidou.mybatisplus.service.IService;
import com.yizhi.application.domain.LecturerResponse;
import com.yizhi.lecturer.application.vo.*;

import java.util.List;

/**
 * <p>
 * 讲师详情表 服务类
 * </p>
 *
 * @author lingye
 * @since 2019-04-17
 */
public interface ILecturerService extends IService<Lecturer> {
    /**
     * 添加讲师库
     *
     * @param lecturerVO 讲师库参数
     * @return
     * @author lilingye
     */
    LecturerVO addLecturer(LecturerVO lecturerVO);

    /**
     * 修改讲师库
     *
     * @param lecturerVO
     * @return
     */
    boolean updateLecturer(LecturerVO lecturerVO);

    /**
     * 删除讲师库
     *
     * @param ids
     * @return
     */
    LecturerResponse delLecturer(List<Long> ids);

    /**
     * 讲师的操作
     *
     * @param lecturerSortVO
     * @return
     */
    boolean sortLecturer(LecturerSortVO lecturerSortVO);

    /**
     * 获取讲师详情的接口
     *
     * @param id
     * @return
     */
    LecturerVO queryLecturerById(Long id);

    /**
     * 讲师列表
     *
     * @param name
     * @param mobile
     * @param sourceCode
     * @param pageNo
     * @param pageSize
     * @return
     */
    Page<LecturerListVO> queryLecturer(String name, String mobile,
                                       String sourceCode,
                                       Integer pageNo, Integer pageSize);

    /**
     * 交银康联 接口
     * @param lecturerApiVoParam
     * @return
     */
    Page<LecturerApiVo> findLecturer(LecturerApiVoParam lecturerApiVoParam);

    /**
     * 导出讲师列表的excel
     *
     * @return
     */
    List<LecturerListVO> exportLecturer(String keywords, String sourceCode, String mobile);

    /**
     * 通过id获取讲师的详情
     *
     * @param id
     * @return
     */
    Lecturer getLecturerById(Long id);

    /**
     * 添加模块查询讲师列表
     *
     * @param name
     * @param pageNo
     * @param pageSize
     * @param ids
     * @return
     */
    Page<TemplateLecturerVO> queryLecturerByTemplateId(String name,
                                                       Integer pageNo, Integer pageSize, List<Long> ids);

    /**
     * 学员端获取讲师详情
     *
     * @param id 讲师Id
     * @return
     */
    LecturerDetailVO queryLecturerByLecturerId(Long id);

    /**
     * 查询讲师不存在
     *
     * @param code
     * @param name
     * @return
     */
    Lecturer getLecturerByCodeAndName(String code, String name, Long siteId);

    /**
     * 讲师库启用停用
     * @param lecturer
     * @return
     */
    LecturerResponse upOrDownLecturer(Lecturer lecturer);

}
