package com.yizhi.application.service.impl;

import com.baomidou.mybatisplus.mapper.EntityWrapper;
import com.baomidou.mybatisplus.service.impl.ServiceImpl;
import com.yizhi.application.orm.util.QueryUtil;
import com.yizhi.core.application.context.ContextHolder;
import com.yizhi.core.application.context.RequestContext;
import com.yizhi.application.domain.LecturerGrade;
import com.yizhi.application.mapper.LecturerGradeMapper;
import com.yizhi.application.orm.id.IdGenerator;
import com.yizhi.application.service.ILecturerGradeService;
import com.yizhi.lecturer.application.vo.LecturerGradeVO;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.Date;
import java.util.List;

/**
 * <p>
 * 讲师级别表 服务实现类
 * </p>
 *
 * @author lingye
 * @since 2019-11-05
 */
@Service
public class LecturerGradeServiceImpl extends ServiceImpl<LecturerGradeMapper, LecturerGrade> implements ILecturerGradeService {
    @Autowired
    private IdGenerator idGenerator;

    /**
     * 新增讲师级别
     *
     * @param lecturerGradeVO
     * @return
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public LecturerGradeVO add(LecturerGradeVO lecturerGradeVO) {
        RequestContext context = ContextHolder.get();
        LecturerGrade lecturerGrade = new LecturerGrade();

        BeanUtils.copyProperties(lecturerGradeVO, lecturerGrade);
        Long id = idGenerator.generate();
        lecturerGrade.setCompanyId(context.getCompanyId());
        lecturerGrade.setSiteId(context.getSiteId());
        Integer gradeCode = this.baseMapper.selectMaxCode(context.getSiteId());
        Integer sort = this.baseMapper.selectMaxSort(context.getSiteId());
        Integer code = 0;
        if (gradeCode != null) {
            code = gradeCode >= 10 ? gradeCode + 1 : 10;
        } else {
            code = 10;
        }
        lecturerGrade.setSort(sort == 0 ? 1 : sort + 1);
        lecturerGrade.setCode(code.toString());
        lecturerGrade.setCreateById(context.getAccountId());
        lecturerGrade.setId(id);
        this.insert(lecturerGrade);
        lecturerGradeVO.setId(id);
        lecturerGradeVO.setCode(code.toString());

        return lecturerGradeVO;
    }

    /**
     * 修改讲师级别
     *
     * @param lecturerGradeVO
     * @return
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public LecturerGradeVO update(LecturerGradeVO lecturerGradeVO) {
        RequestContext context = ContextHolder.get();
        LecturerGrade lecturerGrade = new LecturerGrade();
        BeanUtils.copyProperties(lecturerGradeVO, lecturerGrade);
        lecturerGrade.setUpdateById(context.getAccountId());
        lecturerGrade.setUpdateTime(new Date());
        this.updateById(lecturerGrade);
        return lecturerGradeVO;
    }

    /**
     * 讲师级别列表
     *
     * @return
     */
    @Override
    public List<LecturerGrade> list(Integer status) {
        RequestContext context = ContextHolder.get();
        LecturerGrade lecturerGrade = new LecturerGrade();
        lecturerGrade.setSiteId(context.getSiteId());
        lecturerGrade.setStatus(status);
        lecturerGrade.setDelFlag(0);
        EntityWrapper<LecturerGrade> condition = QueryUtil.condition(lecturerGrade);
        condition.orderBy("sort", true);
        return this.selectList(condition);
    }

}
