package com.yizhi.application.service.impl;

import com.alibaba.fastjson.JSON;
import com.yizhi.application.orm.util.QueryUtil;
import com.yizhi.core.application.context.ContextHolder;
import com.yizhi.core.application.context.RequestContext;
import com.yizhi.application.domain.Lecturer;
import com.yizhi.application.domain.LecturerGrade;
import com.yizhi.application.domain.LecturerRelation;
import com.yizhi.course.application.feign.CourseClient;
import com.yizhi.course.application.feign.OfflineCourseClient;
import com.yizhi.course.application.vo.CourseVo;
import com.yizhi.course.application.vo.OfflineCourseVo;
import com.yizhi.course.application.vo.domain.CourseEntityVo;
import com.yizhi.lecturer.application.enums.DeleteFlag;
import com.yizhi.lecturer.application.enums.ImportError;
import com.yizhi.core.application.exception.BizException;
import com.yizhi.core.application.log.LogQueue;
import com.yizhi.core.application.log.TaskLogDetailEvent;
import com.yizhi.system.application.model.ImportModel;
import com.yizhi.application.service.ILecturerGradeService;
import com.yizhi.application.service.ILecturerImportService;
import com.yizhi.application.service.ILecturerRelationService;
import com.yizhi.application.service.ILecturerService;
import com.yizhi.lecturer.application.vo.LecturerCourseImport;
import com.yizhi.lecturer.application.vo.LecturerScoreImport;
import com.yizhi.util.application.constant.ReturnCode;
import com.yizhi.util.application.regex.RegexUtil;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.ArrayList;
import java.util.List;

/**
 * <p>
 * 讲师详情表 服务类
 * </p>
 *
 * @author lingye
 * @since 2019年11月12日
 */
@Service
public class LecturerImportServiceImpl implements ILecturerImportService {

    @Autowired
    private ILecturerService lecturerService;

    @Autowired
    private CourseClient courseClient;

    @Autowired
    private OfflineCourseClient offlineCourseClient;

    @Autowired
    private ILecturerGradeService lecturerGradeService;


    @Autowired
    private ILecturerRelationService lecturerRelationService;

    /*@Autowired
    private RabbitTemplate rabbitTemplate;*/

    public final String COURSE = "线上";
    public final String OFFLINE_COURSE = "线下";


    private static final Logger logger = LoggerFactory.getLogger(LecturerClassificationServiceImpl.class);

    /**
     * 评分导入
     *
     * @return
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public List<LecturerScoreImport> importScore(ImportModel importModel, Integer type, RequestContext context) {
        logger.info("+++++++++++++++++++++++++++++++++");
        logger.info("站点id：" + context);
        List<LecturerScoreImport> models = importModel.getData();
        Long taskId = importModel.getTaskId();

        List<TaskLogDetailEvent> failEvents = new ArrayList<TaskLogDetailEvent>();
        if (CollectionUtils.isNotEmpty(models)) {
            List<Lecturer> lecturers = new ArrayList<>();
            List<String> codes = new ArrayList<>();
            Lecturer lecturer = new Lecturer();
            for (int j = 2; j < models.size(); j++) {
                int i = j + 2;
                LecturerScoreImport model = models.get(j);
                String code = model.getCode();
                if (StringUtils.isEmpty(code)) {
                    model.setResult(ImportError.CODE_EMPTY);
                    failEvents.add(new TaskLogDetailEvent(taskId, "第" + i + "行:" + ImportError.CODE_EMPTY));
                    continue;
                } else {
                    if (codes.contains(code.trim())) {
                        model.setResult(ImportError.REPEAT_CODE);
                        failEvents.add(new TaskLogDetailEvent(taskId, "第" + i + "行:" + ImportError.REPEAT_CODE));
                        continue;
                    }
                    codes.add(code.trim());
                }
                String score = model.getScore();
                logger.info("score:----" + score);

                lecturer = lecturerService.getLecturerByCodeAndName(code.trim(), null, context.getSiteId());
                logger.info("返回讲师：" + lecturer);
                if (lecturer == null) {
                    model.setResult(ImportError.LECTURER_NOT_EXIST);
                    failEvents.add(new TaskLogDetailEvent(taskId, "第" + i + "行:" + ImportError.LECTURER_NOT_EXIST));
                    continue;
                } else {
                    if (type == 1) {
                        if (StringUtils.isEmpty(score)) {
                            model.setResult(ImportError.SCORE_EMPTY);
                            failEvents.add(new TaskLogDetailEvent(taskId, "第" + i + "行:" + ImportError.SCORE_EMPTY));
                            continue;
                        }
                        try {
                            Double importScore = Double.valueOf(score);
                            logger.info("importScore:{}",importScore);
                            int result = importScore.compareTo(5.0);
                            logger.info("result:{}",result);
                            if (result == 1){
                                    model.setResult(ImportError.SCORE_FORMAT_ERROR);
                                    failEvents.add(new TaskLogDetailEvent(taskId, "第" + i + "行:" + ImportError.SCORE_FORMAT_ERROR));
                                    continue;
                            } else {
                                if (!RegexUtil.lecturerRegex(score)) {
                                    model.setResult(ImportError.SCORE_FORMAT_ERROR);
                                    failEvents.add(new TaskLogDetailEvent(taskId, "第" + i + "行:" + ImportError.SCORE_FORMAT_ERROR));
                                    continue;
                                }
                            }

                        } catch (Exception e) {
                            model.setResult(ImportError.SCORE_FORMAT_ERROR);
                            failEvents.add(new TaskLogDetailEvent(taskId, "第" + i + "行:" + ImportError.SCORE_FORMAT_ERROR));
                            continue;
                        }
                        lecturer.setScore(Float.parseFloat(model.getScore()));
                    } else if (type == 2) {
                        if (StringUtils.isEmpty(score)) {
                            model.setResult(ImportError.SCORE_EMPTY);
                            failEvents.add(new TaskLogDetailEvent(taskId, "第" + i + "行:" + ImportError.GRADE_EMPTY));
                            continue;
                        }
                        LecturerGrade lecturerGrade = new LecturerGrade();
                        lecturerGrade.setName(score.trim());
                        lecturerGrade.setSiteId(context.getSiteId());
                        lecturerGrade.setDelFlag(0);
                        LecturerGrade lg = lecturerGradeService.selectOne(QueryUtil.condition(lecturerGrade));
                        if (lg == null) {
                            failEvents.add(new TaskLogDetailEvent(taskId, "第" + i + "行:" + ImportError.GRADE_NOT_EXIST));
                            continue;
                        } else {
                            lecturer.setGrade(lg.getSort());
                            lecturer.setGradeName(score);
                        }


                    }
                    lecturers.add(lecturer);
                }
                model.setResult("");
            }
            logger.info("讲师库列表：" + lecturers);
            if (CollectionUtils.isNotEmpty(lecturers)) {
                boolean b = lecturerService.updateBatchById(lecturers);
                logger.info("修改后的讲师：" + b);
            }

            logger.info("======success！=======");
            models.remove(0);
            models.remove(0);
            //错误的信息发送给任务详情表
            if (CollectionUtils.isNotEmpty(failEvents)) {
              //  rabbitTemplate.convertAndSend(LogQueue.BATCH_TASK_LOG_DETAIL_QUEUE, failEvents);
            }

        } else {
            logger.info(ReturnCode.RESOURCE_NOT_FOUND.getMsg());
//            throw new BizException(ReturnCode.RESOURCE_NOT_FOUND.getCode(), ReturnCode.RESOURCE_NOT_FOUND.getMsg());

        }


        return models;
    }

    /**
     * 讲师级别导入
     *
     * @param importModel
     * @return
     */
    @Override
    public List<LecturerScoreImport> importGrade(ImportModel importModel) {

        RequestContext context = ContextHolder.get();
        List<LecturerScoreImport> models = importModel.getData();
        Long taskId = importModel.getTaskId();


        if (CollectionUtils.isNotEmpty(models)) {
            List<TaskLogDetailEvent> failEvents = new ArrayList<TaskLogDetailEvent>();
            List<Lecturer> lecturers = new ArrayList<>();
            Lecturer lecturer = new Lecturer();
            for (int j = 1; j <= models.size() - 1; j++) {
                int i = j + 2;
                LecturerScoreImport model = models.get(j);
                String code = model.getCode();
                if (StringUtils.isEmpty(code)) {
                    model.setResult(ImportError.CODE_EMPTY);
                    failEvents.add(new TaskLogDetailEvent(taskId, "第" + i + "行:" + ImportError.CODE_EMPTY));
                    continue;
                }
                String name = model.getName();
                if (StringUtils.isEmpty(name)) {
                    model.setResult(ImportError.NAME_EMPTY);
                    failEvents.add(new TaskLogDetailEvent(taskId, "第" + i + "行:" + ImportError.NAME_EMPTY));
                    continue;
                }
                String score = model.getScore();
                if (StringUtils.isEmpty(score)) {
                    model.setResult(ImportError.SCORE_EMPTY);
                    failEvents.add(new TaskLogDetailEvent(taskId, "第" + i + "行:" + ImportError.SCORE_EMPTY));
                    continue;
                }

                lecturer = lecturerService.getLecturerByCodeAndName(code, name, ContextHolder.get().getSiteId());
                if (lecturer == null) {
                    model.setResult(ImportError.LECTURER_NOT_EXIST);
                    failEvents.add(new TaskLogDetailEvent(taskId, "第" + i + "行:" + ImportError.LECTURER_NOT_EXIST));
                    continue;
                } else {
                    lecturer.setScore(Float.parseFloat(model.getScore()));
                    lecturers.add(lecturer);
                }
            }
            if (CollectionUtils.isNotEmpty(lecturers)) {
                if (!lecturerService.updateBatchById(lecturers)) {
                    throw new BizException(ReturnCode.SAVE_FAIL.getCode(), ReturnCode.SAVE_FAIL.getMsg());
                }
            }
            return models;
        } else {
            throw new BizException(ReturnCode.RESOURCE_NOT_FOUND.getCode(), ReturnCode.RESOURCE_NOT_FOUND.getMsg());
        }
    }

    /**
     * 导入课程
     *
     * @param importModel
     * @return
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public List<LecturerCourseImport> importCourse(ImportModel importModel, RequestContext context) {

//        RequestContext context = ContextHolder.get();
        logger.info("上下文信息：" + JSON.toJSONString(context));
        List<LecturerCourseImport> models = importModel.getData();
        Long taskId = importModel.getTaskId();

        if (CollectionUtils.isNotEmpty(models)) {
            List<TaskLogDetailEvent> failEvents = new ArrayList<TaskLogDetailEvent>();
            List<LecturerRelation> lecturerRelations = new ArrayList<LecturerRelation>();
            List<String> codes = new ArrayList<String>();
            for (int j = 2; j < models.size(); j++) {
                int i = j + 2;
                LecturerCourseImport model = models.get(j);
                String code = model.getCode();
                if (StringUtils.isEmpty(code)) {
                    model.setResult(ImportError.CODE_EMPTY);
                    failEvents.add(new TaskLogDetailEvent(taskId, "第" + i + "行:" + ImportError.CODE_EMPTY));
                    continue;
                }
//                else {
//                    if (codes.contains(code.trim())) {
//                        model.setResult(ImportError.REPEAT_CODE);
//                        failEvents.add(new TaskLogDetailEvent(taskId, "第" + i + "行:" + ImportError.REPEAT_CODE));
//                        continue;
//                    }
//                    codes.add(code.trim());
//                }
                String courseCode = model.getCourseCode();
                logger.info("课程code："+courseCode);
                if (StringUtils.isEmpty(code)) {
                    model.setResult(ImportError.COURSE_CODE_NOT_EXIST);
                    failEvents.add(new TaskLogDetailEvent(taskId, "第" + i + "行:" + ImportError.COURSE_CODE_NOT_EXIST));
                    continue;
                }

                String type = model.getType();
                if (StringUtils.isEmpty(type)) {
                    model.setResult(ImportError.COURSE_TYPE_ERROR);
                    failEvents.add(new TaskLogDetailEvent(taskId, "第" + i + "行:" + ImportError.COURSE_TYPE_ERROR));
                    continue;
                }
                if (COURSE.equals(type.trim()) || OFFLINE_COURSE.equals(type.trim())) {
                    logger.info("课程类型正确");
                } else {
                    model.setResult(ImportError.COURSE_TYPE_ERROR);
                    failEvents.add(new TaskLogDetailEvent(taskId, "第" + i + "行:" + ImportError.COURSE_TYPE_ERROR));
                    continue;
                }

                Lecturer lecturer = lecturerService.getLecturerByCodeAndName(code, null, context.getSiteId());
                if (lecturer == null) {
                    model.setResult(ImportError.LECTURER_NOT_EXIST);
                    failEvents.add(new TaskLogDetailEvent(taskId, "第" + i + "行:" + ImportError.LECTURER_NOT_EXIST));
                    continue;
                } else {
                    Long lecturerId = lecturer.getId();
                    LecturerRelation lecturerRelation = new LecturerRelation();
                    if (type.equals(COURSE)) {
                        Long siteId = context.getSiteId();
                        logger.info("参数id："+siteId);
                        CourseEntityVo course = courseClient.getCourseByCode(model.getCourseCode().trim(), siteId, context.getCompanyId());
                        logger.info("查询的课程：" + course);
                        if (course != null) {
                            lecturerRelation.setLecturerId(lecturerId);
                            lecturerRelation.setRelationId(course.getId());

                            lecturerRelation.setDelFlag(DeleteFlag.NO.ordinal());
                            LecturerRelation lr = lecturerRelationService.selectOne(QueryUtil.condition(lecturerRelation));
                            if (lr != null){
                                model.setResult(ImportError.COURSE_EXIST);
                                failEvents.add(new TaskLogDetailEvent(taskId, "第" + i + "行:" + ImportError.COURSE_EXIST));
                                continue;
                            }

                            lecturerRelation.setRelationName(course.getName());
                            lecturerRelation.setType(1);

                        } else {
                            model.setResult(ImportError.COURSE_NOT_EXIST);
                            failEvents.add(new TaskLogDetailEvent(taskId, "第" + i + "行:" + ImportError.COURSE_NOT_EXIST));
                            continue;
                        }
                    } else if (type.equals(OFFLINE_COURSE)) {
                        Long siteId = context.getSiteId();
                        logger.info("参数id："+siteId);
                        OfflineCourseVo offlineCourse = offlineCourseClient.getOfflineCourseByCode(model.getCourseCode().trim(), siteId);
                        if (offlineCourse != null) {
                            lecturerRelationService.selectOne(QueryUtil.condition(lecturerRelation));
                            lecturerRelation.setRelationId(offlineCourse.getId());
                            lecturerRelation.setLecturerId(lecturerId);
                            lecturerRelation.setDelFlag(DeleteFlag.NO.ordinal());
                            LecturerRelation lr = lecturerRelationService.selectOne(QueryUtil.condition(lecturerRelation));
                            if (lr != null){
                                model.setResult(ImportError.COURSE_EXIST);
                                failEvents.add(new TaskLogDetailEvent(taskId, "第" + i + "行:" + ImportError.COURSE_EXIST));
                                continue;
                            }
                            lecturerRelation.setRelationName(offlineCourse.getName());
                            lecturerRelation.setType(2);


                        } else {
                            model.setResult(ImportError.COURSE_NOT_EXIST);
                            failEvents.add(new TaskLogDetailEvent(taskId, "第" + i + "行:" + ImportError.COURSE_NOT_EXIST));
                            continue;
                        }
                    }
                    lecturerRelations.add(lecturerRelation);
                }
                model.setResult("");

            }
            boolean flag = false;
            if (CollectionUtils.isNotEmpty(lecturerRelations)) {
                flag = lecturerRelationService.insertBatch(lecturerRelations);
            }
            logger.info("新增关系:" + flag);
            logger.info("======success！=======");
            models.remove(0);
            models.remove(0);
            //错误的信息发送给任务详情表
            if (CollectionUtils.isNotEmpty(failEvents)) {
                // rabbitTemplate.convertAndSend(LogQueue.BATCH_TASK_LOG_DETAIL_QUEUE, failEvents);
            }
        } else {
            logger.info(ReturnCode.RESOURCE_NOT_FOUND.getMsg());
        }
        return models;
    }

}
