package com.yizhi.application.service.impl;

import com.baomidou.mybatisplus.plugins.Page;
import com.baomidou.mybatisplus.service.impl.ServiceImpl;
import com.yizhi.application.domain.LecturerRelation;
import com.yizhi.lecturer.application.enums.DeleteFlag;
import com.yizhi.application.mapper.LecturerRelationMapper;
import com.yizhi.application.orm.util.QueryUtil;
import com.yizhi.application.service.ILecturerRelationService;
import com.yizhi.lecturer.application.vo.RelationVO;
import com.yizhi.lecturer.application.vo.RelationVOList;
import lombok.extern.log4j.Log4j2;
import org.apache.commons.collections.CollectionUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;

import static java.util.stream.Collectors.toList;

/**
 * <p>
 * 讲师关联课程表 服务实现类
 * </p>
 *
 * @author lilingye
 * @since 2019-05-06
 */
@Service
@Log4j2
public class LecturerRelationServiceImpl extends ServiceImpl<LecturerRelationMapper, LecturerRelation> implements ILecturerRelationService {

    @Autowired
    private LecturerRelationMapper lecturerRelationMapper;

    @Override
    public boolean insertLecturerRelation(List<LecturerRelation> relationVOS, Long lecturerId) {
        log.info("参数签：" + relationVOS);
        List<LecturerRelation> lecturerRelations = relationVOS.parallelStream().map(lecturerRelationVO -> {
            LecturerRelation lr = new LecturerRelation();
            BeanUtils.copyProperties(lecturerRelationVO, lr);
            lr.setLecturerId(lecturerId);
            return lr;
        }).collect(toList());

        log.info("参数后：" + lecturerRelations);
        return this.insertBatch(lecturerRelations);
    }

    @Override
    public List<RelationVO> getRelationsById(Long id) {
        List<RelationVO> relationVOS = new ArrayList<>();
        LecturerRelation lecturerRelation = new LecturerRelation();
        lecturerRelation.setLecturerId(id);
        lecturerRelation.setDelFlag(DeleteFlag.NO.ordinal());
        List<LecturerRelation> lecturerRelations = this.selectList(QueryUtil.condition(lecturerRelation));
        if (CollectionUtils.isNotEmpty(lecturerRelations)) {
            relationVOS = lecturerRelations.parallelStream().map(lr -> {
                RelationVO relationVO = new RelationVO();
                relationVO.setRelationId(lr.getRelationId());
                relationVO.setRelationName(lr.getRelationName());
                relationVO.setType(lr.getType());
                return relationVO;
            }).collect(toList());
        }
        return relationVOS;
    }

    @Override
    public boolean updateLecturerRelation(List<LecturerRelation> relationVOS, Long lecturerId) {
        LecturerRelation lecturerRelation = new LecturerRelation();
        lecturerRelation.setLecturerId(lecturerId);
        lecturerRelation.setDelFlag(DeleteFlag.NO.ordinal());
        List<LecturerRelation> lecturerRelations =
                this.selectList(QueryUtil.condition(lecturerRelation));

        if (CollectionUtils.isNotEmpty(lecturerRelations)) {
            lecturerRelations = lecturerRelations.parallelStream().map(lr -> {
                lr.setDelFlag(DeleteFlag.YES.ordinal());
                return lr;
            }).collect(toList());
            // 删除原来的
            this.updateBatchById(lecturerRelations);
        }
        if (CollectionUtils.isNotEmpty(relationVOS)) {
            //新增
            this.insertLecturerRelation(relationVOS, lecturerId);
        }

        return true;
    }

    @Override
    public Page<RelationVOList> getRelationListByLecturerId(Long id, Integer pageNo, Integer pageSize) {

        Page<RelationVOList> page = new Page<>();
        page.setCurrent(pageNo);
        page.setSize(pageSize);
        List<RelationVOList> relationVOS = lecturerRelationMapper.getRelationListByLecturerId(page, id);
        page.setRecords(relationVOS);
        return page;

    }
}
