package com.yizhi.application.service.impl;

import com.baomidou.mybatisplus.plugins.Page;
import com.baomidou.mybatisplus.service.impl.ServiceImpl;
import com.yizhi.application.orm.hierarchicalauthorization.HQueryUtil;
import com.yizhi.application.orm.id.IdGenerator;
import com.yizhi.core.application.context.ContextHolder;
import com.yizhi.core.application.context.RequestContext;
import com.yizhi.application.domain.*;
import com.yizhi.application.service.*;
import com.yizhi.lecturer.application.enums.DeleteFlag;
import com.yizhi.lecturer.application.enums.LecturerEnum;
import com.yizhi.application.mapper.LecturerMapper;
import com.yizhi.application.orm.util.QueryUtil;
import com.yizhi.lecturer.application.vo.*;
import com.yizhi.lecturer.application.vo.domain.LecturerClassificationVo;
import com.yizhi.lecturer.application.vo.domain.LecturerRelationVo;
import com.yizhi.site.application.feign.PortalManagePCFeignClients;
import com.yizhi.site.application.vo.domain.ProtalPlateVo;
import lombok.extern.log4j.Log4j2;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import static java.util.stream.Collectors.toList;

/**
 * <p>
 * 讲师详情表 服务实现类
 * </p>
 *
 * @author lingye
 * @since 2019-04-17
 */
@Service

@Log4j2
public class LecturerServiceImpl extends ServiceImpl<LecturerMapper, Lecturer> implements ILecturerService {

    @Autowired
    private IdGenerator idGenerator;

    @Autowired
    private ILecturerService lecturerService;

    @Autowired
    private ILecturerRelationService lecturerRelationService;

    @Autowired
    private ILecturerResumeService lecturerResumeService;

    @Autowired
    private LecturerMapper lecturerMapper;

    @Autowired
    private ILecturerKeywordsService lecturerKeywordsService;

    @Autowired
    private ITrLecturerClassificationService trLecturerClassificationService;


    @Autowired
    private ILecturerClassificationService lecturerClassificationService;

    @Autowired
    private ILecturerInfoService lecturerInfoService;

    @Autowired
    private ILecturerGradeService lecturerGradeService;

    @Autowired
    PortalManagePCFeignClients portalManagePCFeignClient;

    /**
     *
     * @param lecturerVO 讲师库参数
     * @return
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public LecturerVO addLecturer(LecturerVO lecturerVO) {

        RequestContext context = ContextHolder.get();

        Lecturer lecturer = new Lecturer();

        BeanUtils.copyProperties(lecturerVO, lecturer);
        Long lecturerId = idGenerator.generate();
        log.info("讲师id：" + lecturerId);

        lecturer.setId(lecturerId);
        lecturer.setCreateById(context.getAccountId());
        lecturer.setUpdateById(context.getAccountId());
        lecturer.setSiteId(context.getSiteId());
        lecturer.setCompanyId(context.getCompanyId());

        String code = this.baseMapper.queryLecturerCodexSort(context.getSiteId());

        String lecturerNo = getLecturerNo(code);
        lecturer.setCode(lecturerNo);

        // 分类新增
        List<LecturerClassificationVo> lecturerClassificationVos = lecturerVO.getClassificationList();
        if (CollectionUtils.isNotEmpty(lecturerClassificationVos)) {
            List<TrLecturerClassification> classificationList = lecturerClassificationVos.parallelStream().map(
                    lc -> {
                        TrLecturerClassification tlc = new TrLecturerClassification();
                        tlc.setId(idGenerator.generate());
                        tlc.setLecturerId(lecturerId);
                        tlc.setClassificationId(lc.getId());
                        return tlc;
                    }
            ).collect(toList());
            trLecturerClassificationService.insertBatch(classificationList);
        }

        // 讲师简历
        if (StringUtils.isNotEmpty(lecturerVO.getResume())) {
            LecturerResume lecturerResume = new LecturerResume();
            Long lrId = idGenerator.generate();
            lecturerResume.setId(lrId);
            lecturerResume.setLecturerId(lecturerId);
            lecturerResume.setResume(lecturerVO.getResume());
            lecturerResume.setCreateById(context.getAccountId());
            lecturerResume.setUpdateById(context.getAccountId());
            boolean resumeInsert = lecturerResumeService.insert(lecturerResume);
            log.info("讲师简历参数：" + lecturerResume);
            log.info("讲师简历结果：" + resumeInsert);
        }

        if (CollectionUtils.isNotEmpty(lecturerVO.getKeywords())) {
            List<LecturerKeywords> lkList = lecturerVO.getKeywords().parallelStream().map(
                    keyWords -> {
                        LecturerKeywords lecturerKeywords = new LecturerKeywords();
                        lecturerKeywords.setId(idGenerator.generate());
                        lecturerKeywords.setLecturerId(lecturerId);
                        lecturerKeywords.setKeywords(keyWords);
                        return lecturerKeywords;
                    }
            ).collect(toList());
            lecturerKeywordsService.insertBatch(lkList);
        }

        // 排序
        int lastedSort = lecturerMapper.queryLecturerMaxSort(context.getSiteId());
        lecturer.setSort(lastedSort + DeleteFlag.YES.ordinal());
        lecturer.setDelFlag(DeleteFlag.NO.ordinal());
        lecturer.setId(lecturerId);
        log.info("请求的参数：" + lecturer);
        this.insert(lecturer);
        lecturerVO.setId(lecturerId);
        return lecturerVO;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public boolean updateLecturer(LecturerVO lecturerVO) {
        RequestContext context = ContextHolder.get();
        Date currentDate = new Date();
        Lecturer lecturer = new Lecturer();


        // 讲师库主表id
        Long lecturerId = lecturerVO.getId();
        lecturer.setUpdateTime(currentDate);
        lecturer.setUpdateById(context.getAccountId());

//        if (CollectionUtils.isNotEmpty(lecturerVO.getLecturerRelationVoVOS())) {
//            // 删除管理课程
//            LecturerRelationVo lr = new LecturerRelationVo();
//            lr.setLecturerId(lecturerId);
//
//            List<LecturerRelationVo> relations = lecturerRelationService.selectList(QueryUtil.condition(lr));
//            if (CollectionUtils.isNotEmpty(relations)) {
//                relations = relations.parallelStream().map(lecturerRelation -> {
//                    lecturerRelation.setDelFlag(DeleteFlag.YES.ordinal());
//                    return lecturerRelation;
//
//                }).collect(toList());
//
//                // 删除关联课程/或培训
//                boolean delResult = lecturerRelationService.updateBatchById(relations);
//                log.info("删除的结果" + delResult);
//            }
//
//
//            // 新增关联课程
//            List<LecturerRelationVo> lecturerRelationVoVOS = lecturerVO.getLecturerRelationVoVOS();
//            if (CollectionUtils.isNotEmpty(lecturerRelationVoVOS)) {
//                // 讲师库关联的课程
//                boolean addResult = lecturerRelationService.insertLecturerRelation(lecturerRelationVoVOS, lecturerId);
//                log.info("新增关联的结果:" + addResult);
//            }
//        }
        // 删除关键字

        List<String> keywords = lecturerVO.getKeywords();

        LecturerKeywords lecturerKeywords = new LecturerKeywords();
        lecturerKeywords.setLecturerId(lecturerVO.getId());
        List<LecturerKeywords> lecturerKeywordsList = lecturerKeywordsService.selectList(QueryUtil.condition(lecturerKeywords));
        log.info("删除关键字");
        for (LecturerKeywords lw : lecturerKeywordsList) {
            lecturerKeywordsService.deleteById(lw);
        }

        // 新增关键字
        if (CollectionUtils.isNotEmpty(keywords)) {
            List<LecturerKeywords> lkList = lecturerVO.getKeywords().parallelStream().map(
                    keyWords -> {
                        LecturerKeywords words = new LecturerKeywords();
                        words.setId(idGenerator.generate());
                        words.setLecturerId(lecturerVO.getId());
                        words.setKeywords(keyWords);
                        return words;
                    }
            ).collect(toList());
            lecturerKeywordsService.insertBatch(lkList);
        }
        // 讲师库分类
        List<LecturerClassificationVo> lecturerClassificationVos = lecturerVO.getClassificationList();

        TrLecturerClassification lc = new TrLecturerClassification();
        lc.setLecturerId(lecturerVO.getId());
        List<TrLecturerClassification> lcList = trLecturerClassificationService.selectList(QueryUtil.condition(lc));
        log.info("删除分类关系");
        for (TrLecturerClassification tlc : lcList) {
            trLecturerClassificationService.deleteById(tlc);
        }
        if (CollectionUtils.isNotEmpty(lecturerClassificationVos)) {
            // 新增分类关系
            List<TrLecturerClassification> classificationList = lecturerClassificationVos.parallelStream().map(
                    lct -> {
                        TrLecturerClassification tlc = new TrLecturerClassification();
                        tlc.setId(idGenerator.generate());
                        tlc.setLecturerId(lecturerId);
                        tlc.setClassificationId(lct.getId());
                        return tlc;
                    }
            ).collect(toList());
            trLecturerClassificationService.insertBatch(classificationList);
        }
        // 讲师简历
        log.info("参数：" + lecturerVO);
        String resume = lecturerVO.getResume();

        log.info("参数：" + resume);
        if (StringUtils.isNotEmpty(resume)) {
            LecturerResume resumeParam = new LecturerResume();
            resumeParam.setLecturerId(lecturerId);
            resumeParam.setDelFlag(DeleteFlag.NO.ordinal());
            LecturerResume lecturerResume = lecturerResumeService.selectOne(QueryUtil.condition(resumeParam));
            if (lecturerResume != null) {
                // 修改
                lecturerResume.setResume(resume);
                lecturerResume.setUpdateTime(currentDate);
                lecturerResume.setUpdateById(context.getAccountId());
                lecturerResumeService.updateById(lecturerResume);

            } else {
                // 新增
                resumeParam.setCreateTime(currentDate);
                resumeParam.setCreateById(context.getAccountId());
                resumeParam.setResume(lecturerVO.getResume());
                resumeParam.setId(idGenerator.generate());
                lecturerResumeService.insert(resumeParam);

            }


        }

        BeanUtils.copyProperties(lecturerVO, lecturer);

        return this.updateById(lecturer);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public LecturerResponse delLecturer(List<Long> ids) {
        StringBuffer sb = new StringBuffer();

        LecturerResponse lecturerResponse = new LecturerResponse();
        List<Lecturer> lecturerList = new ArrayList<>(16);
        Integer count = 0;
        StringBuffer sbMsg = new StringBuffer();
        for (Long id:ids) {
            Lecturer lecturer = this.selectById(id);
            ProtalPlateVo protalPlate = new ProtalPlateVo();
            protalPlate.setLinkContent(id);
            protalPlate.setState(DeleteFlag.YES.ordinal());
            ProtalPlateVo plate = portalManagePCFeignClient.getPlate(protalPlate);
            if (plate != null) {
                sbMsg.append(lecturer.getLecturerName());
                sbMsg.append(",");
                continue;
            }

            lecturer.setDelFlag(DeleteFlag.YES.ordinal());
            if (lecturer.getStatus()==1) {
                sb.append(lecturer.getLecturerName());
                sb.append(",");
            } else {
                count++;
                lecturerList.add(lecturer);
            }
        }

        // 删除讲师库表
        if (CollectionUtils.isNotEmpty(lecturerList)) {
            this.updateBatchById(lecturerList);
        }

        LecturerRelation lr;
        for (Lecturer l : lecturerList) {
            // 删除相关联的课程/培训
            lr = new LecturerRelation();
            lr.setLecturerId(l.getId());
            List<LecturerRelation> lecturerRelations = lecturerRelationService.selectList(QueryUtil.condition(lr));
            if (CollectionUtils.isNotEmpty(lecturerRelations)) {
                List<LecturerRelation> lecturerRelationList = lecturerRelations.parallelStream().map(lecturerRelation -> {
                    lecturerRelation.setDelFlag(DeleteFlag.YES.ordinal());
                    return lecturerRelation;
                }).collect(toList());
                lecturerRelationService.updateBatchById(lecturerRelationList);
            }

            // 个人简历删除
            LecturerResume lecturerResume = new LecturerResume();
            lecturerResume.setDelFlag(DeleteFlag.YES.ordinal());
            lecturerResume.setLecturerId(l.getId());
            List<LecturerResume> lecturerResumes = lecturerResumeService.selectList(QueryUtil.condition(lecturerResume));
            if (CollectionUtils.isNotEmpty(lecturerResumes)) {
                List<LecturerResume> lecturerResumeList = lecturerResumes.parallelStream().map(lre -> {
                    lre.setDelFlag(DeleteFlag.YES.ordinal());
                    return lre;
                }).collect(toList());
                lecturerResumeService.updateBatchById(lecturerResumeList);
            }
            // 关键字删除
            LecturerKeywords lecturerKeywords = new LecturerKeywords();
            lecturerKeywords.setLecturerId(l.getId());
            List<LecturerKeywords> lecturerKeywordsList = lecturerKeywordsService.selectList(QueryUtil.condition(lecturerKeywords));

            if (CollectionUtils.isNotEmpty(lecturerKeywordsList)) {
                log.info("删除关键字");
                for (LecturerKeywords lw : lecturerKeywordsList) {
                    lecturerKeywordsService.deleteById(lw);
                }
            }
        }
        String msg = sb.toString()+sbMsg.toString();

        if (StringUtils.isNotEmpty(msg)){

            String rMsg = "共有"+count+"位讲师被删除！其中："+ msg.substring(0, msg.length() - 1)+"为启用状态的讲师，不能删除。";
            if (StringUtils.isNotEmpty(sbMsg)) {
                String bMsg = sbMsg+"需下架首页推荐模块，才能删除！";
                lecturerResponse.setMsg(bMsg+rMsg);
            } else {
                lecturerResponse.setMsg(rMsg);
            }
            lecturerResponse.setCode("01");
        } else {
            lecturerResponse.setMsg("删除成功！");
            lecturerResponse.setCode("02");
        }
        return lecturerResponse;
    }

    @Override
    public boolean sortLecturer(LecturerSortVO lecturerSortVO) {

        RequestContext context = ContextHolder.get();
//        context.setSiteId(1314L);
//        context.setCompanyId(1314L);
        String code = lecturerSortVO.getCode();
        Long id = lecturerSortVO.getId();

        Lecturer currentLecturer = this.selectById(id);
        int currentSort = currentLecturer.getSort();

        if (code.equals(LecturerEnum.UP.getKey())) {
            //上移
            Lecturer beforeSortLecturer = lecturerMapper.getLecturerBeforeSort(context.getSiteId(), currentSort);
            if (beforeSortLecturer != null) {
                int beforeSort = beforeSortLecturer.getSort();

                beforeSortLecturer.setSort(currentSort);
                this.updateById(beforeSortLecturer);

                currentLecturer.setSort(beforeSort);
                this.updateById(currentLecturer);
            }


        } else if (code.equals(LecturerEnum.DOWN.getKey())) {
            //下移
            Lecturer beforeSortLecturer = lecturerMapper.getLecturerAfterSort(context.getSiteId(), currentSort);
            if (beforeSortLecturer != null) {
                int beforeSort = beforeSortLecturer.getSort();
                if (beforeSort != DeleteFlag.NO.ordinal()) {
                    beforeSortLecturer.setSort(currentSort);
                    this.updateById(beforeSortLecturer);

                    currentLecturer.setSort(beforeSort);
                    this.updateById(currentLecturer);
                }
            }
        } else if (code.equals(LecturerEnum.TOP.getKey())) {
            //置顶
            List<Lecturer> lecturerBeforeSortList = lecturerMapper.getLecturerBeforeSortList(context.getSiteId(), currentSort);

            lecturerBeforeSortList = lecturerBeforeSortList.parallelStream().map(lecturer -> {
                lecturer.setSort(lecturer.getSort() + DeleteFlag.YES.ordinal());
                return lecturer;
            }).collect(toList());

            this.updateBatchById(lecturerBeforeSortList);
            Lecturer lecturer = new Lecturer();
            lecturer.setId(id);
            Lecturer updateLecturer = lecturerService.selectById(lecturer);
            updateLecturer.setSort(DeleteFlag.NO.ordinal());
            this.updateById(updateLecturer);

        }


        return true;
    }

    @Override
    public LecturerVO queryLecturerById(Long id) {
        LecturerVO lecturerVO = lecturerMapper.getLecturerVOById(id);
        LecturerRelation lr = new LecturerRelation();
        lr.setLecturerId(id);
        lr.setDelFlag(DeleteFlag.NO.ordinal());
        // 关联课程/培训项目
        List<LecturerRelation> lecturerRelations = lecturerRelationService.selectList(QueryUtil.condition(lr));
        List<LecturerRelationVo> lecturerRelationss = new ArrayList<>();
        for (LecturerRelation l:lecturerRelations
        ) {
            LecturerRelationVo lecturerRelationVo=new LecturerRelationVo();
            BeanUtils.copyProperties(l,lecturerRelationVo);
            lecturerRelationss.add(lecturerRelationVo);
        }
        if (CollectionUtils.isNotEmpty(lecturerRelations)) {
            lecturerVO.setLecturerRelationVOS(lecturerRelationss);
        }
        // 简历
        LecturerResume lecturerResume = new LecturerResume();
        lecturerResume.setLecturerId(id);
        lecturerResume.setDelFlag(DeleteFlag.NO.ordinal());
        LecturerResume ls = lecturerResumeService.selectOne(QueryUtil.condition(lecturerResume));
        if (ls != null) {
            lecturerVO.setResumeId(ls.getId());
            lecturerVO.setResume(ls.getResume());
        }
        //关键字
        LecturerKeywords lecturerKeywords = new LecturerKeywords();
        lecturerKeywords.setLecturerId(id);
        List<LecturerKeywords> lecturerKeywordsList = lecturerKeywordsService.selectList(QueryUtil.condition(lecturerKeywords));
        List<String> keywordList = new ArrayList<>();
        if (CollectionUtils.isNotEmpty(lecturerKeywordsList)) {
            keywordList = lecturerKeywordsList.parallelStream().map(lk -> {
                return lk.getKeywords();
            }).collect(toList());
            lecturerVO.setKeywords(keywordList);
        }
        // 分类查询
        TrLecturerClassification trLecturerClassification = new TrLecturerClassification();
        trLecturerClassification.setLecturerId(id);
        List<TrLecturerClassification> trLecturerClassifications = trLecturerClassificationService.selectList(QueryUtil.condition(trLecturerClassification));
        List<LecturerClassification> classifications = trLecturerClassifications.parallelStream().map(
                tr -> {
                    LecturerClassification trc = lecturerClassificationService.selectById(tr.getClassificationId());

                    return trc;
                }
        ).collect(toList());

        List<LecturerClassificationVO> classificationVOS = classifications.parallelStream().map(
                tr -> {
                    LecturerClassificationVO vo = new LecturerClassificationVO();
                    BeanUtils.copyProperties(tr, vo);
                    LecturerClassification topClassification = getTopClassification(tr);
                    vo.setTopId(topClassification.getId());
                    vo.setTopName(topClassification.getName());
                    return vo;
                }
        ).collect(toList());
        // 分类
//        lecturerVO.setClassificationList(classifications);
        lecturerVO.setClassificationVOList(classificationVOS);
        return lecturerVO;
    }

    public LecturerClassification getTopClassification(LecturerClassification lc) {
        LecturerClassification lecturerClassification = new LecturerClassification();

        if (lc.getLayer() == 0) {
            return lc;
        } else {
            String[] strArray = lc.getParentIds().split(",");
            Long id = Long.valueOf(strArray[1]);
            return lecturerClassification.selectById(id);
        }
    }

    @Override
    public Page<LecturerListVO> queryLecturer(String name, String mobile, String sourceCode, Integer pageNo, Integer pageSize) {
        HQueryUtil.startHQ(LecturerListVO.class);
        Page<LecturerListVO> page = new Page<>();
        page.setCurrent(pageNo);
        page.setSize(pageSize);
        RequestContext context = ContextHolder.get();
        List<LecturerListVO> lecturerListVOList = lecturerMapper.getLecturerList(page, name, mobile, sourceCode, context.getSiteId());
        page.setRecords(lecturerListVOList);
        return page;
    }

    @Override
    public Page<LecturerApiVo> findLecturer(LecturerApiVoParam  lecturerApiVoParam) {
        Page<LecturerApiVo> page = new Page<>();
        page.setCurrent(lecturerApiVoParam.getPageNo());
        page.setSize(lecturerApiVoParam.getPageSize());
        RequestContext context = ContextHolder.get();
        List<String> ids = lecturerApiVoParam.getIds();
        ids = CollectionUtils.isNotEmpty(ids)?ids:null;
        List<LecturerApiVo> lecturerListVOList = lecturerMapper.selectLecturerList(page, context.getCompanyId(),context.getSiteId(),ids);
        page.setRecords(lecturerListVOList);
        return page;
    }

    @Override
    public List<LecturerListVO> exportLecturer(String keywords, String sourceCode, String mobile) {
        RequestContext context = ContextHolder.get();
        Long siteId = context.getSiteId();
        return lecturerMapper.exportLecturer(siteId, keywords, sourceCode, mobile);
    }

    @Override
    public Lecturer getLecturerById(Long id) {
        return this.selectById(id);
    }

    @Override
    public Page<TemplateLecturerVO> queryLecturerByTemplateId(String name, Integer pageNo, Integer pageSize, List<Long> ids) {
        Page<TemplateLecturerVO> page = new Page<>();
        page.setCurrent(pageNo);
        page.setSize(pageSize);
        RequestContext context = ContextHolder.get();
        List<TemplateLecturerVO> lecturerListVOList = lecturerMapper.queryLecturerList(page, name, context.getSiteId(), ids);
        page.setRecords(lecturerListVOList);
        return page;
    }

    @Override
    public LecturerDetailVO queryLecturerByLecturerId(Long id) {
        Lecturer lecturer=selectById(id);
        LecturerDetailVO detailVO=lecturerMapper.getStuLecturerDetail(id);
        if (lecturer!= null&& lecturer.getCompanyName()!=null){
            detailVO.setCompanyName(lecturer.getCompanyName());
        }
        return detailVO;

        //初始化返回参数
//        LecturerDetailVO lecturerDetailVO = new LecturerDetailVO();
//        LecturerVo lecturer = this.selectById(id);
//        BeanUtils.copyProperties(lecturer, lecturerDetailVO);
//        // 简历
//        LecturerResumeVo lecturerResume = new LecturerResumeVo();
//        lecturerResume.setLecturerId(id);
//        lecturerResume.setDelFlag(DeleteFlag.NO.ordinal());
//        LecturerResumeVo ls = lecturerResumeService.selectOne(QueryUtil.condition(lecturerResume));
//        if (ls != null) {
//            lecturerDetailVO.setResumeId(ls.getId());
//            lecturerDetailVO.setResume(ls.getResume());
//        }

        //讲师分类
//        TrLecturerClassificationVo trLecturerClassification = new TrLecturerClassificationVo();
//        trLecturerClassification.setLecturerId(id);
//        List<TrLecturerClassificationVo> trLecturerClassifications = trLecturerClassificationService.selectList(QueryUtil.condition(trLecturerClassification));

//        List<LecturerClassificationVo> classificationList = trLecturerClassifications.parallelStream().map(trlc -> {
//            return lecturerClassificationService.selectById(trlc.getClassificationId());
//        }).collect(toList());
//        lecturerDetailVO.setClassifications(classificationList);

        //关键字
//        LecturerKeywordsVo lecturerKeywords = new LecturerKeywordsVo();
//        lecturerKeywords.setLecturerId(id);
//        List<LecturerKeywordsVo> lecturerKeywordsList = lecturerKeywordsService.selectList(QueryUtil.condition(lecturerKeywords));
//        List<String> keywords = null;
//        if (CollectionUtils.isNotEmpty(lecturerKeywordsList)) {
//            keywords = lecturerKeywordsList.stream().map(LecturerKeywordsVo::getKeywords).collect(toList());
//        }
//        lecturerDetailVO.setKeywords(keywords);

//        LecturerInfoVo lecturerInfo = new LecturerInfoVo();
//        lecturerInfo.setSiteId(ContextHolder.get().getSiteId());
//        lecturerInfo.setDelFlag(DeleteFlag.NO.ordinal());
//        lecturerInfo = lecturerInfoService.selectOne(QueryUtil.condition(lecturerInfo));
//        String templateNO = lecturerInfo != null ? lecturerInfo.getTemplateNo() : "1";
//        lecturerDetailVO.setTemplateNo(templateNO);
        // 讲师级名称
//        LecturerGradeVo lecturerGrade = new LecturerGradeVo();
//        lecturerGrade.setSort(lecturer.getGrade());
//        if (lecturer.getGrade()!=null) {
//            lecturerGrade.setSiteId(ContextHolder.get().getSiteId());
//            lecturerGrade = lecturerGradeService.selectOne(QueryUtil.condition(lecturerGrade));
//            lecturerDetailVO.setGradeName(lecturerGrade != null ? lecturerGrade.getName() : "");
//        } else {
//            lecturerDetailVO.setGradeName("");
//        }


//        return lecturerDetailVO;
    }

    /**
     * @param code
     * @param name
     * @return
     */
    @Override
    public Lecturer getLecturerByCodeAndName(String code, String name, Long siteId) {

        Lecturer lecturer = new Lecturer();
        lecturer.setCode(code);
        lecturer.setDelFlag(DeleteFlag.NO.ordinal());
        lecturer.setSiteId(siteId);
        Lecturer le = this.selectOne(QueryUtil.condition(lecturer));
        return le;
    }

    /**
     * 讲师库启用和停用
     *
     * @param lecturer
     * @return
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public LecturerResponse upOrDownLecturer(Lecturer lecturer) {
        LecturerResponse lecturerResponse = new LecturerResponse();

        if (DeleteFlag.NO.ordinal() == lecturer.getStatus()) {
            ProtalPlateVo protalPlate = new ProtalPlateVo();
            protalPlate.setLinkContent(lecturer.getId());
            protalPlate.setState(DeleteFlag.YES.ordinal());
            ProtalPlateVo plate = portalManagePCFeignClient.getPlate(protalPlate);
            if (plate != null) {
                lecturerResponse.setCode("fail");
                lecturerResponse.setMsg("该讲师已在平台首页讲师模块展示，无法禁用!");
                return lecturerResponse;
            } else {
                this.updateById(lecturer);
                lecturerResponse.setCode("success");
                return lecturerResponse;
            }

        } else {
            this.updateById(lecturer);
            return lecturerResponse;
        }

    }

    public static String getLecturerNo(String equipmentNo) {
        String newEquipmentNo = "T000001";

        if (equipmentNo != null && !equipmentNo.isEmpty()) {
            int newEquipment = Integer.parseInt(equipmentNo) + 1;
            newEquipmentNo = String.format("T" + "%06d", newEquipment);
        }

        return newEquipmentNo;
    }
}
